/* Function curreg */

func curreg( im, shape, &xcen, &ycen, &params, color=, thick= )
/* DOCUMENT s = curreg( im, shape, >xcen, >ycen, >params, color=, thick= )

     A function to make a cursor marked region in an image.

     Returns an array with image indices.

     The shape can be "c" for circular (default), "a" for annulus, or "b" for box.
     When "d" is given an annulus with center (xcen,ycen) is defined.

     Returned in 'params':
           circular : [xcen, ycen, radius]
           annulus  : [xcen, ycen, radius1, radius2]
           box      : [x1, y1, x2, y2]
*/
{
   dms = dimsof(im);
   if( is_void(shape) ) shape = "c";
   if( typeof(shape) != "string" ) error,"CURREG: Second argument must be a string";
   sh = strpart(shape,1:1);
   if( sh == "c" ) { // circular region
      cent = curmark1(prompt="Mark the center ...");
      xcen = cent(1);
      ycen = cent(2);
      rad  = curmark1(prompt="Mark the periphery ...");
      v = span(0,2*pi,50);
      write,"     - calculating ...";
      r = sqrt((xcen-rad(1))^2 + (ycen-rad(2))^2);
      params = [xcen, ycen, r];
      d = distances(dms(2),dms(3),xcen,ycen);
      w = where(d < r);
      oplot,xcen+r*cos(v),ycen+r*sin(v),color=color,thick=thick;
   } else if( sh == "a" || sh == "d" ) { // annular region
      if( sh == "a" ) {
         cent = curmark1(prompt="Mark the center ...");
         xcen = cent(1);
         ycen = cent(2);
      }
      v = span(0,2*pi,50);
      write,"     - calculating ...";
      d = distances(dms(2),dms(3),xcen,ycen);

      rad  = curmark1(prompt="Mark the inner periphery ...");
      r1 = sqrt((xcen-rad(1))^2 + (ycen-rad(2))^2);
      oplot,xcen+r1*cos(v),ycen+r1*sin(v),color=color,thick=thick;
      rad  = curmark1(prompt="Mark the outer periphery ...");
      r2 = sqrt((xcen-rad(1))^2 + (ycen-rad(2))^2);
      params = [xcen, ycen, r1, r2];
      oplot,xcen+r2*cos(v),ycen+r2*sin(v),color=color,thick=thick;
      w = where( d > r1 & d < r2 );
   } else {          // box region
      corners = curmark1(prompt="Mark the box corners ...",style=2);
      x1 = min(corners(1:3:2));
      x2 = max(corners(1:3:2));
      xcen = 0.5*(x1+x2);
      y1 = min(corners(2:4:2));
      y2 = max(corners(2:4:2));
      params = [x1, y1, x2, y2 ];
      ycen = 0.5*(y1+y2);
      xnet = indgen(dms(2))(,-:1:dms(3));
      ynet = indgen(dms(3))(-:1:dms(2),);
      write,"     - calculating ...";
      w = where(xnet > x1 & xnet < x2 & ynet > y1 & ynet < y2 );
      oplot,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],color=color,thick=thick;
   }
   return w;
}

/* Function selreg */

func selreg( im, shape, params )
/* DOCUMENT s = selreg( im, shape, params )

     A function to return selected pixel from a predefined region.

     Returns an array with image indices.

     The shape can be "c" for circular (default), "a" for annulus, or "b" for box.
     When "d" is given an annulus with center (xcen,ycen) is defined.

*/
{
   dms = dimsof(im);
   if( typeof(shape) != "string" ) error,"SELREG: Second argument must be a string";
   sh = strpart(shape,1:1);
   if( sh == "c" ) { // circular region
      // params is [xcen, ycen, radius]
      d = distances(dms(2),dms(3),params(1),params(2));
      w = where(d < params(3));
   } else if( sh == "a" || sh == "d" ) { // annular region
      // params is [xcen, ycen, radius1, radius2]
      d = distances(dms(2),dms(3),params(1),params(2));
      w = where( d > params(3) & d < params(4) );
   } else if( sh == "b" ) {          // box region
      // params is [x1, y1, x2, y2 ]
      xnet = indgen(dms(2))(,-:1:dms(3));
      ynet = indgen(dms(3))(-:1:dms(2),);
      w = where(xnet > params(1) & xnet < params(3) & ynet > params(2) & ynet < params(4) );
   } else { // unknown or void "v"
      return [];
   }
   return w;
}
